/****************************************************************************
** Meta object code from reading C++ file 'tickerboard.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.9.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../widget/tickerboard.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tickerboard.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.9.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11TickerBoardE_t {};
} // unnamed namespace

template <> constexpr inline auto TickerBoard::qt_create_metaobjectdata<qt_meta_tag_ZN11TickerBoardE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TickerBoard",
        "charChanged",
        "",
        "newChar",
        "updateRequired",
        "isReady",
        "keyPressed",
        "key",
        "startTicker",
        "wasPaused",
        "pauseTicker",
        "txt",
        "setTicker",
        "extendTicker",
        "seperator",
        "getNewChar",
        "setErrorSelection",
        "clearErrorSelection",
        "progress"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'charChanged'
        QtMocHelpers::SignalData<void(QChar)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QChar, 3 },
        }}),
        // Signal 'updateRequired'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'isReady'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'keyPressed'
        QtMocHelpers::SignalData<void(QChar)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QChar, 7 },
        }}),
        // Slot 'startTicker'
        QtMocHelpers::SlotData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Slot 'startTicker'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'pauseTicker'
        QtMocHelpers::SlotData<void(QString)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'pauseTicker'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'setTicker'
        QtMocHelpers::SlotData<void(QString)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'setTicker'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'extendTicker'
        QtMocHelpers::SlotData<void(QString, QString)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 }, { QMetaType::QString, 14 },
        }}),
        // Slot 'extendTicker'
        QtMocHelpers::SlotData<void(QString)>(13, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'extendTicker'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'getNewChar'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setErrorSelection'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clearErrorSelection'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'progress'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TickerBoard, qt_meta_tag_ZN11TickerBoardE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TickerBoard::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11TickerBoardE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11TickerBoardE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11TickerBoardE_t>.metaTypes,
    nullptr
} };

void TickerBoard::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TickerBoard *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->charChanged((*reinterpret_cast< std::add_pointer_t<QChar>>(_a[1]))); break;
        case 1: _t->updateRequired(); break;
        case 2: _t->isReady(); break;
        case 3: _t->keyPressed((*reinterpret_cast< std::add_pointer_t<QChar>>(_a[1]))); break;
        case 4: _t->startTicker((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->startTicker(); break;
        case 6: _t->pauseTicker((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->pauseTicker(); break;
        case 8: _t->setTicker((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->setTicker(); break;
        case 10: _t->extendTicker((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QString>>(_a[2]))); break;
        case 11: _t->extendTicker((*reinterpret_cast< std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->extendTicker(); break;
        case 13: _t->getNewChar(); break;
        case 14: _t->setErrorSelection(); break;
        case 15: _t->clearErrorSelection(); break;
        case 16: _t->progress(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TickerBoard::*)(QChar )>(_a, &TickerBoard::charChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TickerBoard::*)()>(_a, &TickerBoard::updateRequired, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TickerBoard::*)()>(_a, &TickerBoard::isReady, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TickerBoard::*)(QChar )>(_a, &TickerBoard::keyPressed, 3))
            return;
    }
}

const QMetaObject *TickerBoard::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TickerBoard::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11TickerBoardE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int TickerBoard::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void TickerBoard::charChanged(QChar _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void TickerBoard::updateRequired()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void TickerBoard::isReady()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void TickerBoard::keyPressed(QChar _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
