// This file is autogenerated. Changes will be overwritten.
#include "4XVY5JCB3V/moc_abcrainwidget.cpp"
#include "4XVY5JCB3V/moc_charball.cpp"
#include "BFRA4CB3AK/moc_chartablesql.cpp"
#include "BFRA4CB3AK/moc_lessontablesql.cpp"
#include "UZBZ344ZLF/moc_errormessage.cpp"
#include "UZBZ344ZLF/moc_evaluationwidget.cpp"
#include "UZBZ344ZLF/moc_fingerwidget.cpp"
#include "UZBZ344ZLF/moc_helpbrowser.cpp"
#include "UZBZ344ZLF/moc_illustrationdialog.cpp"
#include "UZBZ344ZLF/moc_keyboard.cpp"
#include "UZBZ344ZLF/moc_lessondialog.cpp"
#include "UZBZ344ZLF/moc_lessonprintdialog.cpp"
#include "UZBZ344ZLF/moc_lessonresult.cpp"
#include "UZBZ344ZLF/moc_mainwindow.cpp"
#include "UZBZ344ZLF/moc_numpad.cpp"
#include "UZBZ344ZLF/moc_progressionwidget.cpp"
#include "UZBZ344ZLF/moc_regexpdialog.cpp"
#include "UZBZ344ZLF/moc_settings.cpp"
#include "UZBZ344ZLF/moc_startwidget.cpp"
#include "UZBZ344ZLF/moc_statusbar.cpp"
#include "UZBZ344ZLF/moc_tickerboard.cpp"
#include "UZBZ344ZLF/moc_trainingwidget.cpp"
