/********************************************************************************
** Form generated from reading UI file 'settings.ui'
**
** Created by: Qt User Interface Compiler version 6.9.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SETTINGS_H
#define UI_SETTINGS_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_Settings
{
public:
    QGridLayout *gridLayout_3;
    QTabWidget *tabWidget;
    QWidget *training;
    QVBoxLayout *verticalLayout_5;
    QGroupBox *groupBox_4;
    QVBoxLayout *verticalLayout_3;
    QGridLayout *gridLayout_7;
    QPushButton *buttonBackground;
    QPushButton *buttonFontColor;
    QLabel *label_9;
    QPushButton *buttonFont;
    QPushButton *buttonCursor;
    QLabel *label_4;
    QLabel *label_7;
    QLabel *label_8;
    QVBoxLayout *verticalLayout_6;
    QLabel *label_3;
    QGridLayout *gridLayout_4;
    QLabel *label_5;
    QSlider *tickerSpeed;
    QLabel *label_6;
    QGroupBox *groupBox_5;
    QGridLayout *gridLayout_2;
    QCheckBox *checkMetronome;
    QSpacerItem *horizontalSpacer;
    QSpinBox *spinBoxMetronome;
    QWidget *language;
    QHBoxLayout *horizontalLayout_2;
    QGroupBox *groupBox_3;
    QGridLayout *gridLayout;
    QComboBox *comboBoxTrainingLessons;
    QPushButton *buttonLayoutAdvanced;
    QLabel *label_2;
    QComboBox *comboBoxKeyboardLayout;
    QLabel *labelLessonNotice;
    QLabel *label;
    QSpacerItem *verticalSpacer;
    QWidget *results;
    QHBoxLayout *horizontalLayout;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_4;
    QPushButton *buttonResetLessons;
    QPushButton *buttonResetRecordedChars;
    QWidget *other;
    QVBoxLayout *verticalLayout;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout_2;
    QCheckBox *checkWelcome;
    QCheckBox *checkIntelligence;
    QCheckBox *checkLimitLesson;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *Settings)
    {
        if (Settings->objectName().isEmpty())
            Settings->setObjectName("Settings");
        Settings->resize(643, 430);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(Settings->sizePolicy().hasHeightForWidth());
        Settings->setSizePolicy(sizePolicy);
        gridLayout_3 = new QGridLayout(Settings);
        gridLayout_3->setObjectName("gridLayout_3");
        tabWidget = new QTabWidget(Settings);
        tabWidget->setObjectName("tabWidget");
        tabWidget->setLayoutDirection(Qt::LeftToRight);
        tabWidget->setTabPosition(QTabWidget::North);
        tabWidget->setTabShape(QTabWidget::Rounded);
        training = new QWidget();
        training->setObjectName("training");
        verticalLayout_5 = new QVBoxLayout(training);
        verticalLayout_5->setObjectName("verticalLayout_5");
        groupBox_4 = new QGroupBox(training);
        groupBox_4->setObjectName("groupBox_4");
        verticalLayout_3 = new QVBoxLayout(groupBox_4);
        verticalLayout_3->setObjectName("verticalLayout_3");
        gridLayout_7 = new QGridLayout();
        gridLayout_7->setObjectName("gridLayout_7");
        buttonBackground = new QPushButton(groupBox_4);
        buttonBackground->setObjectName("buttonBackground");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(buttonBackground->sizePolicy().hasHeightForWidth());
        buttonBackground->setSizePolicy(sizePolicy1);

        gridLayout_7->addWidget(buttonBackground, 1, 3, 1, 1);

        buttonFontColor = new QPushButton(groupBox_4);
        buttonFontColor->setObjectName("buttonFontColor");
        sizePolicy1.setHeightForWidth(buttonFontColor->sizePolicy().hasHeightForWidth());
        buttonFontColor->setSizePolicy(sizePolicy1);

        gridLayout_7->addWidget(buttonFontColor, 1, 2, 1, 1);

        label_9 = new QLabel(groupBox_4);
        label_9->setObjectName("label_9");

        gridLayout_7->addWidget(label_9, 0, 4, 1, 1);

        buttonFont = new QPushButton(groupBox_4);
        buttonFont->setObjectName("buttonFont");
        sizePolicy1.setHeightForWidth(buttonFont->sizePolicy().hasHeightForWidth());
        buttonFont->setSizePolicy(sizePolicy1);

        gridLayout_7->addWidget(buttonFont, 1, 0, 1, 1);

        buttonCursor = new QPushButton(groupBox_4);
        buttonCursor->setObjectName("buttonCursor");
        sizePolicy1.setHeightForWidth(buttonCursor->sizePolicy().hasHeightForWidth());
        buttonCursor->setSizePolicy(sizePolicy1);

        gridLayout_7->addWidget(buttonCursor, 1, 4, 1, 1);

        label_4 = new QLabel(groupBox_4);
        label_4->setObjectName("label_4");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Maximum, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(label_4->sizePolicy().hasHeightForWidth());
        label_4->setSizePolicy(sizePolicy2);

        gridLayout_7->addWidget(label_4, 0, 0, 1, 1);

        label_7 = new QLabel(groupBox_4);
        label_7->setObjectName("label_7");

        gridLayout_7->addWidget(label_7, 0, 2, 1, 1);

        label_8 = new QLabel(groupBox_4);
        label_8->setObjectName("label_8");

        gridLayout_7->addWidget(label_8, 0, 3, 1, 1);


        verticalLayout_3->addLayout(gridLayout_7);

        verticalLayout_6 = new QVBoxLayout();
        verticalLayout_6->setObjectName("verticalLayout_6");
        label_3 = new QLabel(groupBox_4);
        label_3->setObjectName("label_3");
        sizePolicy2.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy2);

        verticalLayout_6->addWidget(label_3);

        gridLayout_4 = new QGridLayout();
        gridLayout_4->setObjectName("gridLayout_4");
        label_5 = new QLabel(groupBox_4);
        label_5->setObjectName("label_5");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(label_5->sizePolicy().hasHeightForWidth());
        label_5->setSizePolicy(sizePolicy3);

        gridLayout_4->addWidget(label_5, 0, 0, 1, 1);

        tickerSpeed = new QSlider(groupBox_4);
        tickerSpeed->setObjectName("tickerSpeed");
        sizePolicy1.setHeightForWidth(tickerSpeed->sizePolicy().hasHeightForWidth());
        tickerSpeed->setSizePolicy(sizePolicy1);
        tickerSpeed->setMaximum(4);
        tickerSpeed->setValue(2);
        tickerSpeed->setOrientation(Qt::Horizontal);

        gridLayout_4->addWidget(tickerSpeed, 0, 1, 1, 1);

        label_6 = new QLabel(groupBox_4);
        label_6->setObjectName("label_6");
        sizePolicy3.setHeightForWidth(label_6->sizePolicy().hasHeightForWidth());
        label_6->setSizePolicy(sizePolicy3);

        gridLayout_4->addWidget(label_6, 0, 2, 1, 1);


        verticalLayout_6->addLayout(gridLayout_4);


        verticalLayout_3->addLayout(verticalLayout_6);


        verticalLayout_5->addWidget(groupBox_4);

        groupBox_5 = new QGroupBox(training);
        groupBox_5->setObjectName("groupBox_5");
        gridLayout_2 = new QGridLayout(groupBox_5);
        gridLayout_2->setObjectName("gridLayout_2");
        checkMetronome = new QCheckBox(groupBox_5);
        checkMetronome->setObjectName("checkMetronome");

        gridLayout_2->addWidget(checkMetronome, 0, 0, 1, 1);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_2->addItem(horizontalSpacer, 0, 2, 1, 1);

        spinBoxMetronome = new QSpinBox(groupBox_5);
        spinBoxMetronome->setObjectName("spinBoxMetronome");
        spinBoxMetronome->setEnabled(false);
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(spinBoxMetronome->sizePolicy().hasHeightForWidth());
        spinBoxMetronome->setSizePolicy(sizePolicy4);
        spinBoxMetronome->setMinimum(1);
        spinBoxMetronome->setMaximum(200);
        spinBoxMetronome->setValue(60);

        gridLayout_2->addWidget(spinBoxMetronome, 0, 1, 1, 1);


        verticalLayout_5->addWidget(groupBox_5);

        tabWidget->addTab(training, QString());
        language = new QWidget();
        language->setObjectName("language");
        horizontalLayout_2 = new QHBoxLayout(language);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        groupBox_3 = new QGroupBox(language);
        groupBox_3->setObjectName("groupBox_3");
        groupBox_3->setFlat(false);
        groupBox_3->setCheckable(false);
        gridLayout = new QGridLayout(groupBox_3);
        gridLayout->setObjectName("gridLayout");
        comboBoxTrainingLessons = new QComboBox(groupBox_3);
        comboBoxTrainingLessons->setObjectName("comboBoxTrainingLessons");

        gridLayout->addWidget(comboBoxTrainingLessons, 4, 0, 1, 1);

        buttonLayoutAdvanced = new QPushButton(groupBox_3);
        buttonLayoutAdvanced->setObjectName("buttonLayoutAdvanced");

        gridLayout->addWidget(buttonLayoutAdvanced, 2, 0, 1, 1);

        label_2 = new QLabel(groupBox_3);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 3, 0, 1, 1);

        comboBoxKeyboardLayout = new QComboBox(groupBox_3);
        comboBoxKeyboardLayout->setObjectName("comboBoxKeyboardLayout");

        gridLayout->addWidget(comboBoxKeyboardLayout, 1, 0, 1, 1);

        labelLessonNotice = new QLabel(groupBox_3);
        labelLessonNotice->setObjectName("labelLessonNotice");
        labelLessonNotice->setEnabled(true);
        QFont font;
        font.setPointSize(10);
        labelLessonNotice->setFont(font);
        labelLessonNotice->setWordWrap(true);

        gridLayout->addWidget(labelLessonNotice, 5, 0, 1, 1);

        label = new QLabel(groupBox_3);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 0, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 6, 0, 1, 1);


        horizontalLayout_2->addWidget(groupBox_3);

        tabWidget->addTab(language, QString());
        results = new QWidget();
        results->setObjectName("results");
        horizontalLayout = new QHBoxLayout(results);
        horizontalLayout->setObjectName("horizontalLayout");
        groupBox = new QGroupBox(results);
        groupBox->setObjectName("groupBox");
        verticalLayout_4 = new QVBoxLayout(groupBox);
        verticalLayout_4->setObjectName("verticalLayout_4");
        buttonResetLessons = new QPushButton(groupBox);
        buttonResetLessons->setObjectName("buttonResetLessons");

        verticalLayout_4->addWidget(buttonResetLessons);

        buttonResetRecordedChars = new QPushButton(groupBox);
        buttonResetRecordedChars->setObjectName("buttonResetRecordedChars");

        verticalLayout_4->addWidget(buttonResetRecordedChars);


        horizontalLayout->addWidget(groupBox);

        tabWidget->addTab(results, QString());
        other = new QWidget();
        other->setObjectName("other");
        verticalLayout = new QVBoxLayout(other);
        verticalLayout->setObjectName("verticalLayout");
        groupBox_2 = new QGroupBox(other);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout_2 = new QVBoxLayout(groupBox_2);
        verticalLayout_2->setObjectName("verticalLayout_2");
        checkWelcome = new QCheckBox(groupBox_2);
        checkWelcome->setObjectName("checkWelcome");

        verticalLayout_2->addWidget(checkWelcome);

        checkIntelligence = new QCheckBox(groupBox_2);
        checkIntelligence->setObjectName("checkIntelligence");

        verticalLayout_2->addWidget(checkIntelligence);

        checkLimitLesson = new QCheckBox(groupBox_2);
        checkLimitLesson->setObjectName("checkLimitLesson");
        sizePolicy4.setHeightForWidth(checkLimitLesson->sizePolicy().hasHeightForWidth());
        checkLimitLesson->setSizePolicy(sizePolicy4);
        checkLimitLesson->setChecked(false);
        checkLimitLesson->setAutoRepeat(false);
        checkLimitLesson->setTristate(false);

        verticalLayout_2->addWidget(checkLimitLesson);


        verticalLayout->addWidget(groupBox_2);

        tabWidget->addTab(other, QString());

        gridLayout_3->addWidget(tabWidget, 2, 0, 1, 1);

        buttonBox = new QDialogButtonBox(Settings);
        buttonBox->setObjectName("buttonBox");
        sizePolicy1.setHeightForWidth(buttonBox->sizePolicy().hasHeightForWidth());
        buttonBox->setSizePolicy(sizePolicy1);
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Help|QDialogButtonBox::Save);
        buttonBox->setCenterButtons(false);

        gridLayout_3->addWidget(buttonBox, 3, 0, 1, 1);


        retranslateUi(Settings);
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, Settings, qOverload<>(&QDialog::reject));

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(Settings);
    } // setupUi

    void retranslateUi(QDialog *Settings)
    {
        Settings->setWindowTitle(QCoreApplication::translate("Settings", "Settings", nullptr));
#if QT_CONFIG(tooltip)
        training->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        groupBox_4->setTitle(QCoreApplication::translate("Settings", "Ticker", nullptr));
#if QT_CONFIG(tooltip)
        buttonBackground->setToolTip(QCoreApplication::translate("Settings", "Here you can select the background color", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonBackground->setText(QString());
#if QT_CONFIG(tooltip)
        buttonFontColor->setToolTip(QCoreApplication::translate("Settings", "Here you can select the font color", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonFontColor->setText(QString());
        label_9->setText(QCoreApplication::translate("Settings", "Cursor:", nullptr));
#if QT_CONFIG(tooltip)
        buttonFont->setToolTip(QCoreApplication::translate("Settings", "Here you can change the font of the ticker (a font size larger than 20 pt is not recommended)", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonFont->setText(QCoreApplication::translate("Settings", "Change &Font", nullptr));
#if QT_CONFIG(tooltip)
        buttonCursor->setToolTip(QCoreApplication::translate("Settings", "Here can select the color of the cursor for the current character", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonCursor->setText(QString());
        label_4->setText(QCoreApplication::translate("Settings", "Font:", nullptr));
        label_7->setText(QCoreApplication::translate("Settings", "Font Color:", nullptr));
        label_8->setText(QCoreApplication::translate("Settings", "Background:", nullptr));
        label_3->setText(QCoreApplication::translate("Settings", "Speed:", nullptr));
        label_5->setText(QCoreApplication::translate("Settings", "Off", nullptr));
#if QT_CONFIG(tooltip)
        tickerSpeed->setToolTip(QCoreApplication::translate("Settings", "Here you can change the speed of the ticker (Slider on the left: Ticker does not move until reaching the end of line. Slider on the right: The ticker moves very fast.)", nullptr));
#endif // QT_CONFIG(tooltip)
        label_6->setText(QCoreApplication::translate("Settings", "Fast", nullptr));
        groupBox_5->setTitle(QCoreApplication::translate("Settings", "Audio Output", nullptr));
#if QT_CONFIG(tooltip)
        checkMetronome->setToolTip(QCoreApplication::translate("Settings", "Here you can activate a metronome", nullptr));
#endif // QT_CONFIG(tooltip)
        checkMetronome->setText(QCoreApplication::translate("Settings", "Metronome:", nullptr));
#if QT_CONFIG(tooltip)
        spinBoxMetronome->setToolTip(QCoreApplication::translate("Settings", "Select how often the metronome sound should appear per minute", nullptr));
#endif // QT_CONFIG(tooltip)
        spinBoxMetronome->setSpecialValueText(QString());
        spinBoxMetronome->setSuffix(QCoreApplication::translate("Settings", " cpm", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(training), QCoreApplication::translate("Settings", "Training", nullptr));
        groupBox_3->setTitle(QCoreApplication::translate("Settings", "Language", nullptr));
        comboBoxTrainingLessons->setPlaceholderText(QString());
        buttonLayoutAdvanced->setText(QCoreApplication::translate("Settings", "Advanced", nullptr));
        label_2->setText(QCoreApplication::translate("Settings", "Training Lessons:", nullptr));
        labelLessonNotice->setText(QCoreApplication::translate("Settings", "The training lessons you have chosen are not suited for the keyboard layout. You can continue but you may have to put aside some keys from the beginning.", nullptr));
        label->setText(QCoreApplication::translate("Settings", "Keyboard Layout:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(language), QCoreApplication::translate("Settings", "Language", nullptr));
        groupBox->setTitle(QCoreApplication::translate("Settings", "User Data", nullptr));
#if QT_CONFIG(tooltip)
        buttonResetLessons->setToolTip(QCoreApplication::translate("Settings", "Here you can reset all saved lesson data (the lessons will be empty as they were after initial installation", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonResetLessons->setText(QCoreApplication::translate("Settings", "Reset &completed lessons", nullptr));
#if QT_CONFIG(tooltip)
        buttonResetRecordedChars->setToolTip(QCoreApplication::translate("Settings", "Here you can reset all recorded keystrokes and typing mistakes (the characters will be empty as they were after initial installation)", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonResetRecordedChars->setText(QCoreApplication::translate("Settings", "Reset all &recorded characters", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(results), QCoreApplication::translate("Settings", "Results", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("Settings", "Windows", nullptr));
#if QT_CONFIG(tooltip)
        checkWelcome->setToolTip(QCoreApplication::translate("Settings", "Here you can decide if an information window with tips is shown at the beginning", nullptr));
#endif // QT_CONFIG(tooltip)
        checkWelcome->setText(QCoreApplication::translate("Settings", "Show welcome message at startup", nullptr));
#if QT_CONFIG(tooltip)
        checkIntelligence->setToolTip(QCoreApplication::translate("Settings", "Here you can define whether a warning window is displayed when an open or own lesson with active intelligence is started", nullptr));
#endif // QT_CONFIG(tooltip)
        checkIntelligence->setText(QCoreApplication::translate("Settings", "Remember enabled intelligence before starting\n"
"an open or own lesson", nullptr));
        checkLimitLesson->setText(QCoreApplication::translate("Settings", "Change duration of the lesson automatically to\n"
"\"Entire lesson\" when disabling the intelligence", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(other), QCoreApplication::translate("Settings", "Other", nullptr));
    } // retranslateUi

};

namespace Ui {
    class Settings: public Ui_Settings {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SETTINGS_H
