/********************************************************************************
** Form generated from reading UI file 'regexpdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.9.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_REGEXPDIALOG_H
#define UI_REGEXPDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_RegExpDialog
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QLineEdit *lineRegExp;
    QLabel *label_4;
    QLabel *label_3;
    QLineEdit *lineReplace;
    QLabel *label_2;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *RegExpDialog)
    {
        if (RegExpDialog->objectName().isEmpty())
            RegExpDialog->setObjectName("RegExpDialog");
        RegExpDialog->resize(652, 337);
        verticalLayout = new QVBoxLayout(RegExpDialog);
        verticalLayout->setObjectName("verticalLayout");
        label = new QLabel(RegExpDialog);
        label->setObjectName("label");

        verticalLayout->addWidget(label);

        lineRegExp = new QLineEdit(RegExpDialog);
        lineRegExp->setObjectName("lineRegExp");

        verticalLayout->addWidget(lineRegExp);

        label_4 = new QLabel(RegExpDialog);
        label_4->setObjectName("label_4");
        QFont font;
        font.setPointSize(10);
        font.setKerning(true);
        label_4->setFont(font);
        label_4->setWordWrap(true);

        verticalLayout->addWidget(label_4);

        label_3 = new QLabel(RegExpDialog);
        label_3->setObjectName("label_3");

        verticalLayout->addWidget(label_3);

        lineReplace = new QLineEdit(RegExpDialog);
        lineReplace->setObjectName("lineReplace");

        verticalLayout->addWidget(lineReplace);

        label_2 = new QLabel(RegExpDialog);
        label_2->setObjectName("label_2");
        QFont font1;
        font1.setPointSize(10);
        label_2->setFont(font1);
        label_2->setWordWrap(true);

        verticalLayout->addWidget(label_2);

        buttonBox = new QDialogButtonBox(RegExpDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::RestoreDefaults|QDialogButtonBox::Save);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(RegExpDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, RegExpDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(RegExpDialog);
    } // setupUi

    void retranslateUi(QDialog *RegExpDialog)
    {
        RegExpDialog->setWindowTitle(QCoreApplication::translate("RegExpDialog", "Filter for the keyboard layout", nullptr));
        label->setText(QCoreApplication::translate("RegExpDialog", "Limitation of characters", nullptr));
        label_4->setText(QCoreApplication::translate("RegExpDialog", "You should try to avoid using characters not supported by your keyboard layout. A filter as a regular expression is applied to all practice texts before the lesson begins. You should only make changes here if you are familiar with regular expressions.", nullptr));
        label_3->setText(QCoreApplication::translate("RegExpDialog", "Replacement Filter", nullptr));
        label_2->setText(QCoreApplication::translate("RegExpDialog", "Filtering unauthorized characters can produce texts that make little sense (e.g., by removing umlauts). You can define replacements that will be applied before the limitation of characters is applied. Please follow the example here that replaces all Germans umlauts and the \303\237 symbol:\n"
"ae=ae,oe=oe,ue=ue,Ae=Ae,Oe=Oe,Ue=Ue,ss=ss", nullptr));
    } // retranslateUi

};

namespace Ui {
    class RegExpDialog: public Ui_RegExpDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_REGEXPDIALOG_H
